@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015
+;;  Copyright (C) 2013,2014,2015,2016,2017
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -25,6 +25,7 @@
   #:use-module (artanis db)
   #:use-module (artanis fprm)
   #:use-module (artanis ssql)
+  #:use-module (artanis session)
   #:use-module (artanis oht)
   #:use-module (artanis route)
   #:use-module (artanis page)
@@ -32,6 +33,8 @@
   #:use-module (artanis debug)
   #:use-module (artanis third-party csv)
   #:use-module (artanis third-party json)
+  #:use-module (artanis server)
+  #:use-module (artanis websocket)
   #:use-module (artanis version)
   #:use-module (web server)
   #:use-module (srfi srfi-1)
@@ -42,13 +45,13 @@
                throw-auth-needed
                tpl->html
                redirect-to
-               generate-response-with-file
-               emit-response-with-file
                tpl->response
                reject-method
                response-error-emit
                run-after-request!
                run-before-response!
+               emit-response-with-file
+               static-page-emitter
 
                ;; oht module
                get
@@ -71,6 +74,7 @@
                :auth
                :session
                :from-post
+               :websocket
 
                ;; db module
                DB-open
@@ -85,6 +89,7 @@
                connect-db
                make-<connection>
                <connection>?
+               current-connection
 
                ;; fprm module
                map-table-from-DB
@@ -119,7 +124,8 @@
                new-route-context
                route-context?
                get-header
-
+               get-from-qstr
+               
                ;; csv
                make-csv-reader
                csv->xml
@@ -161,8 +167,7 @@
 
                ;; version
                artanis-version)
-  #:export (static-page-emitter
-            result-ref
+  #:export (result-ref
             init-server
             form-tag
             label-tag
@@ -174,28 +179,28 @@
 (define* (result-ref alst key #:key (decode? #t))
   (and=> (assoc-ref alst key) (if decode? uri-decode identity)))
 
-;; When you don't want to use cache, use static-page-emitter.
-(define (static-page-emitter rc)
-  (emit-response-with-file (static-filename (rc-path rc))))
-
 (define (default-route-init statics cache-statics? exclude)
   ;; avoid a common warn
-  (get "/" (lambda () "no index.html but it works!"))
+  (get "/" #:cache #t (lambda () "no index.html but it works!"))
   (let ((srule (format #f "^/.+\\.(~{~a~^|~})$" (lset-difference eq? statics exclude))))
     (if cache-statics?
-        (get srule #:cache 'static (lambda (rc) (:cache rc)))
+        (get srule #:cache '(static) (lambda (rc) (:cache rc)))
         (get srule static-page-emitter))))
 
+(define (check-invalid-config)
+  (when (and (not (linux-version>=? "3.9")) (get-conf '(server multi)))
+    (error "It's required to have Linux-3.9+ to enable server.multi feature!")))
+
 ;; make sure to call init-server at the beginning
 (define* (init-server #:key (statics '(png jpg jpeg ico html js css))
                       (cache-statics? #f) (exclude '()))
   (default-route-init statics cache-statics? exclude)
   (init-hook)
-  (init-config))
-
-(define (check-if-not-run-init-server)
-  ;; Just check if the conf table is empty
-  (is-hash-table-empty? *conf-hash-table*))
+  (init-config)
+  (init-server-core)
+  (check-invalid-config)
+  (sigaction SIGPIPE SIG_IGN)
+  (set! is-init-server-run? #t))
 
 (define* (form-tag #:key (controller #f) (action #f) (method #f)
                    (class #f) (tag-class #f) (tag-id #f) (form-method "get"))
@@ -240,17 +245,23 @@
 (define p-tag (make-general-tag 'p))
 (define div-tag (make-general-tag 'div))
 
-(define (debug-mode:before-request-handler rc body)
+(define (debug-mode:file-monitoring rc body)
   (reload-monitored-files))
 
 (define (init-debug-mode)
   (conf-set! 'debug-mode #t)
-  (init-debug-monitor)
-  (run-after-request! debug-mode:before-request-handler))
+  (cond
+   ((current-toplevel)
+    ;; NOTE: If (current-toplevel) is #f, then it means you're not running under
+    ;;       application folder, then the file monitoring feature is disabled automatically.
+    (init-debug-monitor)
+    (run-after-request! debug-mode:file-monitoring))
+   (else (format (artanis-current-output)
+                 "[WARN] You're not in application folder, the file motoring is disabled!~%"))))
 
 ;; Invalid use-db? must be (dbd username passwd) or #f
-(define* (run #:key (host #f) (port #f) (debug #f) (use-db? #f)
-              (dbd #f) (db-username #f) (db-passwd #f) (db-name #f))
+(define* (run #:key (host #f) (port #f) (debug #f) (use-db? #f) (db-proto #f)
+              (dbd #f) (db-username #f) (db-passwd #f) (db-name #f) (db-addr #f))
   (define (->proper-body-display body)
     (cond
      ((not body) "No body in the request!")
@@ -261,8 +272,12 @@
                      (and=> (string-length body)
                             (lambda (len) (if (> len 100) 100 len))))))
      (((@ (rnrs bytevectors) bytevector?) body) "Body is bytevectors!")
-     (else (throw 'artanis-err 500 "->proper-body-display: Invalid body type!" body))))
-  (when (check-if-not-run-init-server)
+     ((websocket-frame? body)
+      (if debug
+          (print-websocket-frame body)
+          "<websocket-frame>"))
+     (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type!" body))))
+  (when (not is-init-server-run?)
     (error "Sorry, but you have to run (init-server) in the begining of you main program!"))
   (and host (conf-set! '(host addr) host))
   (and port (conf-set! '(host port) port))
@@ -272,16 +287,26 @@
   (when (or use-db? (get-conf 'use-db?))
     (conf-set! 'use-db? #t)
     (display "User wants to use Database, initializing...\n")
-    (init-database-config dbd db-username db-passwd db-name)
+    (init-database-config dbd db-username db-passwd db-name db-addr db-proto)
     (init-DB)
-    (display "DB init done!\n"))
+    (display "DB init done!\n")
+    (when (eq? 'db (get-conf '(session backend)))
+          (session-init)
+          (display "Session with DB backend init done!\n")))
+  (case (get-conf '(session backend))
+    ((db)
+     (when (not (get-conf 'use-db?))
+           (error "Session with DB backend init failed because you didn't enable DB!")))
+    (else
+     (session-init)
+     (format #t "Session with ~:@(~a~) backend init done!~%"
+             (get-conf '(session backend)))))
   (run-hook *before-run-hook*)
   (format #t "~a~%" (current-myhost))
-  (format #t "Anytime you want to Quit just try Ctrl+C, thanks!~%")
-  (run-server
-   (if debug
-       (lambda (r b)
-         (format #t "[Request] ~a~%[Body] ~a~%" r (->proper-body-display b))
-         (server-handler r b))
-       server-handler)
-   'http `(#:host ,(get-conf '(host addr)) #:port ,(get-conf '(host port)))))
+  (format #t "Anytime you want to quit just try Ctrl+C, thanks!~%")
+  (let ((handler (if debug
+                     (lambda (r b)
+                       (format #t "[Request] ~a~%[Body] ~a~%" r (->proper-body-display b))
+                       (server-handler r b))
+                     server-handler)))
+    (establish-http-gateway handler)))
