@@ -69,7 +69,7 @@
   (hash-ref *handlers-table* handler-key))
 
 (define-record-type route-context
-  (make-route-context handler keys regexp request path qt method rhk bt 
+  (make-route-context handler keys regexp request path qt method rhk bt
                       body date cookie set-cookie conn)
   route-context?
   (handler rc-handler rc-handler!) ; reqeust handler
@@ -77,7 +77,7 @@
   (regexp rc-re rc-re!) ; regexp to parse key-bindings
   (request rc-req rc-req!) ; client request
   ;; FIXME: actually we don't need this redundant path,
-  ;;        it's better to get from request. 
+  ;;        it's better to get from request.
   (path rc-path rc-path!) ; path from uri
   (qt rc-qt rc-qt!) ; query table
   ;; FIXME: the current Guile inner server treat HEAD as GET, so we
@@ -86,7 +86,7 @@
   (rhk rc-rhk rc-rhk!) ; rule handler key in handlers-table
   (bt rc-bt rc-bt!) ; bindings table
   (body rc-body rc-body!) ; request body
-  (date rc-mtime rc-mtime!) ; modified time, users need to set it in handler
+  (date rc-mtime rc-mtime!) ; modified time, users may want to set it
   (cookie rc-cookie rc-cookie!) ; the cookie parsed from header string
   (set-cookie rc-set-cookie rc-set-cookie!) ; the cookies needed to be set as response
   ;; auto DB connection doesn't need users to close it, it's auto closed when request is over.
@@ -135,19 +135,22 @@
          (hrc (if handler-key  ; get handler-keys pair
                   (get-handler-rc handler-key)
                   (throw 'artanis-err 404 init-rule-handler-and-keys!
-                         "invalid handler key ~a" handler-key))))
+                         "Client ~a had visited an invalid path ~a"
+                         (REASON-TEXT (remote-info (rc-req rc)))
+                         (REASON-TEXT (rc-path rc))))))
     (rc-handler! rc (handler-rc-handler hrc))
-    (rc-keys! rc (handler-rc-keys hrc))))
+    (rc-keys! rc (reverse (handler-rc-keys hrc)))))
 
 (define (init-rule-path-regexp! rc)
   (rc-re! rc (string->irregex (cdr (rc-rhk rc)))))
 
 ;; init key-bindings table
 (define (init-rule-key-bindings! rc)
-  (let ((m (irregex-search (rc-re rc) (rc-path rc))))
+  (let* ((m (irregex-search (rc-re rc) (rc-path rc)))
+         (num (irregex-match-num-submatches m)))
     (rc-bt! rc
-            (map (lambda (k i) (cons k (irregex-match-substring m i))) 
-                 (rc-keys rc) (iota (irregex-match-num-submatches m) 1)))))
+            (map (lambda (k i) (cons k (irregex-match-substring m i)))
+                 (rc-keys rc) (iota num 1)))))
 
 (define (init-query! rc)
   ;; NOTE: All the prefix/postfix ":" in query/post keys are trimmed.
@@ -157,11 +160,11 @@
   (let ((str (case (rc-method rc)
                ((GET) (uri-query (request-uri (rc-req rc))))
                ;; The accessor of GET and POST should be divided
-               ((POST) #f) ; don't handle post here
-               (else (throw 'artanis-err 405 
+               ((POST PUT DELETE HEAD OPTIONS PATCH) #f) ; don't handle post here
+               (else (throw 'artanis-err 405 init-query!
                             "wrong method for query!" (rc-method rc))))))
     (if str
-        (rc-qt! rc (map (lambda (x) 
+        (rc-qt! rc (map (lambda (x)
                           (map -> (string-split x #\=)))
                         (string-split str #\&)))
         '())))
